------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   IR CLOCK&BIAS BOARD
-- DEVICE ID    :   VIRTEX FPGA (U116)
-- PARENT MODULE:   TOP LEVEL (CB_FPGA)  
-- MODULE       :   BOARD SILICON SERIAL NUMBER (SSN) I/F
-- VERSION      :   1.1
------------------------------------------------------------------------
-- FILE NAME    :   ONEWIRE_IFACE.VHD
-- AUTHOR       :   GUSTAVO RAHMER
-- DATE         :   15/April/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE Webpack 5.1
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- Interface for the Silicon Serial Number Device DS2401 through a 1-Wire Bus.
--
-- It will output the data from the DS2401 byte by byte while data_valid is
-- asserted. Totally 8 bytes of data will show up on the data bus in sequence of:
--      Family code (x01 for DS2401) : 1 byte
--      Serial number (e.g. xABCDEF) : 6 bytes
--      CRC value (based on polynomial = x^8 + x^5 + x^4 + 1) : 1 byte
--
-- The Serial number appears in sn_data after completion of the sequence.
--                
-- It connects to the DS2401 through only one wire (dq), which is a bidirectional
-- data path with internal pullup resistor (about 13KOhm, which is higher than the
-- specification of 5KOhm on the DS2401 datasheet) or an external pull-up resistor.
--
-- It uses a clock divider to generate a slow clock (1MHz) for the 1-Wire Master
-- module from the system clock. Use a generic (CLK_DIV) to specify the divider 
-- ratio for different input clock rate.
--
-- It generates a crc_ok signal which indicates all the data have been received/output
-- and crc checking is OK if the generic (CheckCRC) is turned on.
--
-- THE ORIGINAL SOURCE FOR THIS MODULE AND ITS SUB-MODULES IS THE XILINX WEB REPOSITORY.
-- REFERENCE: APPLICATION NOTE "XAPP198" FROM www.xilinx.com
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- The ports required to connect this module in your specific top level are:
--   SYS_CLK: connect to system clock.
--   SYS_RESET: connect to a strobe signal (active high) to trigger readout.
--              If tied to VCC, readout will start automatically when sys_clk is turned on.
--   CRC_OK: connect to a bit in the status word.
--   SN_DATA: connect to the corresponding input (48 bits) for the SSN on your Personality module
--   DQ: connect to the corresponding pin that goes to the DS2401 chip (bi-directional)
-- 
-- The other two ports (DATA, DATA_VALID) can be left unconnected.
--
-- Files required for this module:
--   + clk_div.vhd
--   + onewire_master.vhd
--      + shreg.vhd
--      + bitReg.vhd
--      + byteReg.vhd
--      + crcreg.vhd
--      + jcounter.vhd
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	      AUTHOR  VERSION  MODIFICATIONS
-- 15/Apr/04    GR      1.0    Generic version adapted from original source.
--                             Added dummy value option.
-- 16/Apr/04    GR      1.1    Personalized for MONSOON C&B board.
--
------------------------------------------------------------------------
-- PORT DEFINITIONS :
--  SYS_CLK    : system clock (40Mhz)
--  SYS_RESET  : active high reset 
--  DQ : connect to the 1-wire bus
--  DATA       : data output
--  DATA_VALID : data output valid (20us strobe)
--  CRC_OK     : crc ok signal (active high)
--  SN_DATA    : parallel output (valid when crc_ok is active)
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Title      : An interface to use the 1-Wire Master
-- File       : onewire_iface.vhd
-- Author     : Davy Huang <Dai.Huang@Xilinx.com>
-- Company    : Xilinx, Inc.
-- Created    : 2001/02/01
-- Last Update: 2001-04-18
-- Copyright  : (c) Xilinx Inc, 2001
-------------------------------------------------------------------------------
-- Uses       : 1-Wire Master (onewire_master.vhd), Clock Divider (clk_div.vhd)
-------------------------------------------------------------------------------
-- Used by    : Any user system connecting to a 1-Wire device
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- synthesis translate_off
-- synopsys translate_off
library unisim;
use unisim.vcomponents.all;
-- synopsys translate_on
-- synthesis translate_on

entity onewire_iface is
    generic (
       CLK_DIV  : integer range 0 to 15 := 9; -- Clock Divider for 40MHz system clock input
       ADD_PULLUP : boolean := true;   -- Add a pullup on the 1-Wire bus
       CheckCRC : boolean := true;   -- Turn on/off CRC Generator
       UseDummy : boolean := false    -- Outputs dummy value for sn_data (instead of the real value)
       );    
    port (
       sys_clk    : in  std_logic;    -- system clock (40Mhz)
       sys_reset  : in  std_logic;    -- active high syn. reset 
       dq         : inout std_logic;  -- connect to the 1-wire bus
       data       : out std_logic_vector(7 downto 0); -- data output
       data_valid : out std_logic;    -- data output valid (20us strobe)
       crc_ok     : out std_logic;    -- crc ok signal (active high)
       sn_data    : out std_logic_vector(47 downto 0) -- parallel output
        );
end onewire_iface;

architecture rtl of onewire_iface is

   component onewire_master
      generic (CheckCRC : boolean);
      port (
             clk_1MHz  : in  std_logic;    
             reset     : in  std_logic;    
             dq        : inout std_logic;
             data      : out std_logic_vector(7 downto 0);
             data_valid: out std_logic;
             crcok     : out std_logic;
             sn_data   : out std_logic_vector(47 downto 0)
                );
    end component;

    component PULLUP   -- pull up resistor component
       port (O : out std_logic);
    end component;
    
    component clk_divider
      generic (CLK_DIV : integer range 0 to 15);  -- Clock Divider
      port (
       reset   : in  std_logic;    -- asynchronous reset
       clk_in  : in  std_logic;    -- system clock 
       clk_out : out std_logic);   -- output a slow clock
    end component;
    
    -- signals for reset    
    signal reset_i : std_logic;

    -- signals for generating clock
    signal clk_1MHz    : std_logic;

    -- signals for crc output
    signal crcok    : std_logic;

    -- signal for real sn data    
    signal sn_data_real : std_logic_vector(47 downto 0);

    -- dummy value for test only
    constant dummy_ssn : std_logic_vector := x"fafafafafafa";

begin


   -----------------------------------------------------------
   -- wiring
   -----------------------------------------------------------
    reset_i <= sys_reset;      
    
    crc_ok  <=  crcok;    

   -----------------------------------------------------------
   -- Generate a 1MHz clock for the onewire_master
   -- using the clock divider
   --
   -----------------------------------------------------------
   --   Table : Generic Settings in Clock Divider Based on
   --           Input Clock Rates 
   -----------------------------------------------------------
   -- Min Input        Max Input     Divider   CLK_DIV 
   -- Clock Freq.     Clock Freq.     Ratio     Value
   --  (MHz)             (MHz) 
   -----------------------------------------------------------
   --    3              5                4         0
   --    5              9                8         1
   --    9             14                12        2
   --   14             18                16        3
   --   18             22                20        4
   --   22             26                24        5
   --   26             30                28        6
   --   30             34                32        7
   --   34             38                36        8
   --   38             42                40        9
   --   42             46                44       10
   --   46             50                48       11
   --   50             54                52       12
   --   54             58                56       13
   --   58             62                60       14
   --   62             80                64       15
   -----------------------------------------------------------
    clkdivider: clk_divider
       generic map 
         ( CLK_DIV => CLK_DIV)  
       port map               
         ( clk_in => sys_clk,
           reset  => reset_i,
           clk_out=> clk_1MHz);

   -----------------------------------------------------------
   -- interface to serial number master    
   -----------------------------------------------------------
    ow_master_i: onewire_master
       generic map
        (  CheckCRC => CheckCRC)
       port map
        (   clk_1MHz  => clk_1MHz,
            reset     => reset_i,
            dq        => dq,
            data      => data,
--          data      => open,   -- leave it open if do not use it            
            sn_data   => sn_data_real, -- comment out for dummy fixed value
--          sn_data   => open,   -- leave it open if do not use it
            data_valid=> data_valid,
            crcok     => crcok);

    sn_data <= sn_data_real when UseDummy = false else
               dummy_ssn;

    -----------------------------------------------------------
    -- Pullup components
    -----------------------------------------------------------
    pullupgen: if ADD_PULLUP = true generate
       pullupSNDQ  : PULLUP port map (O => dq);
    end generate pullupgen;

    
end rtl;
