------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   IR ACQUISITION BOARD
-- DEVICE ID    :   CLOCK&BIAS FPGA U116
-- PARENT MODULE:   TOP LEVEL  
-- MODULE       :   RESET LOGIC FOR CLOCK&BIAS FPGA
-- VERSION      :   4.0 
------------------------------------------------------------------------
-- FILE NAME    :   RESET_LOGIC.VHD
-- AUTHOR       :   PETER MOORE
-- DATE         :   04/29/2004
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 5.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- TAKES AS INPUT THE SEQUENCER BUS RESET AND PRODUCES ONE RESET
-- PULSE LOCAL TO THE DEVICE + A FLAG TO INDICATE THAT RESET HAS OCCURED.
-- ALSO PROVIDES THE BOOT UP RESET PULSE.
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- RESET PULSE DURATION IS 15ms ON BOOT ELSE 1us FOR OTHER SOURCES
-- THE NOMINAL BOOT PERIOD IS 200MS WITH A CONFIG RATE OF 34. 
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	      AUTHOR  VERSION  MODIFICATIONS
-- 04/29/2004   PCM   4.0   ORIGINAL VERSION ADAPTED FROM MASTER CONTROL BOARD SEQ FPGA
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- SYSCLK       : SYSTEM CLOCK SOURCE
-- RST_CMD      : SOFT RESET REQUEST STROBE FROM PAN
-- SYNC_RCVD    : SYNCHRONIZATION WORD RECEIVED FROM PAN
-- ASYNC_FLAG   : AFTER RESET, THIS IS SET TO INDICATE THAT SYNCHRONIZATION WITH PAN MAY HAVE BEEN LOST
-- RESET_OUT    : NOMINAL, LOCAL RESET SIGNAL.
------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity reset_logic is
    Port ( 
        SYSCLK      : in  std_logic;
        RST_CMD     : in  std_logic;
        SYNC_RCVD   : in  std_logic;
        ASYNC_FLAG  : out std_logic;
        RESET_OUT   : out std_logic
    );
end reset_logic;

architecture Behavioral of reset_logic is
    constant BootRstPer : std_logic_vector(19 downto 0) := x"92C70";   -- Period of boot reset pulse (15 ms = 92C70h).
    constant ExtRstPer  : std_logic_vector(19 downto 0) := x"00028";   -- Period of normal reset pulse (1 us = 28h). 
    signal RstPeriod    : std_logic_vector(19 downto 0);
    signal RstCounter   : std_logic_vector(19 downto 0);
    signal BootFlag     : std_logic := '0';
    signal LocalRst     : std_logic; 
begin

------------------------------------------------------------------------
-- Main reset logic process
------------------------------------------------------------------------

    RESET_OUT <= LocalRst;

    RstTimerProc : process (SYSCLK)
    begin
        if (rising_edge(SYSCLK)) then
            if (BootFlag = '0') then            -- Only occurs once after boot process
                BootFlag <= '1';                -- Prevent retriggering until next boot.
                RstPeriod <= BootRstPer;        -- Load the required reset period in
                RstCounter <= (others => '0');  -- Probably redundant.
            else                                -- All other cases except first sysclk edge after boot.
                if (RstCounter < RstPeriod) then    -- While reset period active :-
                    LocalRst <= '1';                -- Fire off a trigger to the async latch logic.
                    RstCounter <= RstCounter + 1;   -- Increment period counter
                else                                -- We've reached the end of our time
                    LocalRst <= '0';                    -- Reset the deed
                    if (RST_CMD = '1') then
                        RstPeriod <= ExtRstPer;         -- We need a reset so load apropriate period
                        RstCounter <= (others => '0');  -- Repeat the performance.
                    end if;
                end if;
            end if;    
        end if;
    end process;

    AsynFlagProc : process (SYSCLK)
    begin
        if (rising_edge(SYSCLK)) then
            if (LocalRst = '1') then
                ASYNC_FLAG <= '1';
            elsif (SYNC_RCVD = '1') then
                ASYNC_FLAG <= '0';
            end if;
        end if;
    end process;

end Behavioral;
